import { HttpService } from '@nestjs/axios';
import { ConfigService } from '@nestjs/config';
import { catchError, firstValueFrom, map } from 'rxjs';
import { Injectable } from '@nestjs/common';
import { RESTResponseBody, RESTError } from '../models/dtos/wrappers/rest-wrapper.class';
import { ISMTP } from '../models/interfaces/smtp.interface';

@Injectable()
export class SMTPService {
    //#region Private Members
    private _baseUrl: string;
    //#endregion

    //#region Constructor
    constructor(private readonly _httpService: HttpService, private readonly _configService: ConfigService) {
        this._baseUrl = this._configService.getOrThrow('DL_SMTP__REST__BASE_URL');
    }
    //#endregion

    //#region Public API
    public async send(subject: string, from: string, to: string, html: string, profileId: string) {
        const requestBody = {
            data: {
                from,
                to: [to],
                subject,
                html,
            },
        };

        const response$ = this._httpService.post<RESTResponseBody<ISMTP>>(`${this._baseUrl}/v1/emails/send`, requestBody, { params: { profileId } });
        const response = await firstValueFrom(response$);

        if (response.data.errors) {
            throw new RESTError('smtp_api_error', response.data.errors);
        }
        if (!response.data || !response.data.result) {
            throw new RESTError('smtp_api_error', 'empty response');
        }

        return response.data.result.data;
    }

    public send$(subject: string, from: string, to: string, html: string, profileId: string) {
        const requestBody = {
            data: {
                from,
                to: [to],
                subject,
                html,
            },
        };

        const response$ = this._httpService.post<RESTResponseBody<ISMTP>>(`${this._baseUrl}/v1/emails/send`, requestBody, { params: { profileId } });
        return response$.pipe(
            map((response) => {
                if (response.data.errors) {
                    throw new RESTError('smtp_api_error', response.data.errors);
                }
                if (!response.data || !response.data.result) {
                    throw new RESTError('smtp_api_error', 'empty response');
                }
                return response.data.result.data;
            }),
            catchError((e) => {
                throw new RESTError('smtp_api_error', e);
            }),
        );
    }
    //#endregion
}
